/*******************************************************************************

   @file    europa_phy_module.h
   @brief   This file contains public constants, structures, and function
            prototypes for the europa PHY access code.

   @copyright Copyright (C) 2019 Cumulus Networks, Inc. All rights reserved

   @remark  This software is subject to the Cumulus Networks End User License
            Agreement available  at the following locations:

   @remark  Internet: https://cumulusnetworks.com/downloads/eula/latest/view/

   @remark  Cumulus Linux systems: /usr/share/cumulus/EULA.txt

*******************************************************************************/

#if !defined (__EUROPA_PHY_MODULE_H_)
#define __EUROPA_PHY_MODULE_H_

#include "xphy.h"

/*------------------------------------------------------------------------------

                                 Constants

------------------------------------------------------------------------------*/

#define DEBUG 0

#define LANE_MAX 4

#define SYS_SIDE  1
#define LINE_SIDE 0

#define EGRESS    0
#define INGRESS   1

/* The number of entries in the linkscan log array */
#define LINKSCAN_LOG_SIZE 100

typedef enum fec_type_e {
    EUROPA_NO_FEC,
    EUROPA_CL74_FEC,
    EUROPA_RS_FEC
} fec_type_t;

#define if_error_msg(rv, fmt, ...) \
    do {\
        int _rv=(rv);\
        if (_rv!=0) { \
            cli_out("Error %s:%d <%s>: rv=%d " fmt,\
                     __FILE__, __LINE__, __func__, _rv, ##__VA_ARGS__);\
            return _rv;\
        }\
    } while(0);

#define if_error(rv) if_error_msg(rv, "\n")

#define debug_print(fmt, ...) \
            do { if (DEBUG) fprintf(stderr, fmt, ##__VA_ARGS__); } while (0)

#define EN_STR(A) ((A) ? "enabled" : "disabled")
#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))


/*------------------------------------------------------------------------------

                                  Structures

------------------------------------------------------------------------------*/

typedef struct europa_phy_id_table_s {
    unsigned int    phy_reg;        /* Register address which contains ID */
    unsigned int    exp_phy_id;     /* Value expected from the ID register */
    unsigned int    data;           /* Value actually read */
    int             phy_type;       /* The phy_type identifier (part num) */
} europa_phy_id_table_t;

/**
   @brief A structure which contains information about a port
          that is participating in linkscan. These are placed in
          a singly linked list.

   @field enabled - boolean which tracks wether the port is
          currently regsitered with the SDK for linkscanning.
   @field sav_enabled - boolean used to restore the previous
          enabled state.
   @field unit/port - The unit and BCM SDK logical port
   @field phy_id - The MDIO address of the PHY for the port
   @field lane_map - A bitmap of the PHY lanes for the port
   @field name - The PHY family name of the port
   @field sys_forced_map - A bitmap of lanes on the system side
          which have been TX squelched by linkscan.
   @field line_forced_map - A bitmap of lanes on the line side
          which have been TX squelched by linkscan.
   @field port_name - The linux interface port name
   The next three fields are indexed by the "side", either
   line(0) or system (1):
   @field if_type - The interface type
   @field auto_neg - Enable autonegotiation
   @field fec - The type of FEC in use
   @field speed - The speed of the port in Mbps
 */
typedef struct linkscan_list_s {
    struct linkscan_list_s *next;
    unsigned char           enabled;
    unsigned char           sav_enabled;
    int                     unit;
    bcm_port_t              port;
    unsigned int            phy_id;
    unsigned int            lane_map;
    char                   *name;
    unsigned int            sys_forced_map;
    unsigned int            line_forced_map;
    char                   *port_name;
    int                     if_type[2];
    int                     auto_neg[2];
    int                     fec[2];
    int                     speed;
} linkscan_list_t;

/**
   @brief A structure which contain the information of one
          linkscan log entry.

   @field unit/port - The unit and BCM SDK logical port
   @field line_link_map - A bitmap of PHY lanes which have link
          on the line side of the PHY for this port.
   @field sys_link_map - A bitmap of PHY lanes which have link
          on the system side of the PHY for this port.
 */
typedef struct linkscan_log_s {
    int unit;
    int port;
    unsigned int line_link_map;
    unsigned int sys_link_map;
} linkscan_log_t;

/**
   @brief A structure which contains all of the statistics kept
          during linkscanning, including the log.

   @field linkscan_calls - The number of times the linkscan
          function has been executed.
   @field not_found - The number of times the linkscan function
          has been called and an entry for the port was not
          found in the linked list.
   @field line_errors - The number of errors returned by the SDK
          for line-side SDK calls.
   @field sys_errors - The number of errors returned by the SDK
          for system-side SDK calls.
   @field read_idx - The index in the log for the oldest entry.
   @field write_idx - The index in the log where the next entry
          will be stored.
   @field overflows - The number of entries in the log which
          have been overwritten.
   @field speed_errors - The number errors evaluating/setting
          link speed.
   @field log - A circular log storing information about each
          time the linkscan function was called.
 */
typedef struct linkscan_stats_s {
    int linkscan_calls;
    int not_found;
    int line_errors;
    int sys_errors;
    int read_idx;
    int write_idx;
    int overflows;
    int speed_errors;
    linkscan_log_t ls_log[LINKSCAN_LOG_SIZE];
} linkscan_stats_t;


/*------------------------------------------------------------------------------

                                  Global Data

------------------------------------------------------------------------------*/

phy_functions_t europa_functions;


#endif /** __EUROPA_PHY_MODULE_H_ */
