/*******************************************************************************

   @file    xphy.h
   @brief   This file contains public constants, structures, and function
            prototypes for the xphy command line processing.

   @copyright Copyright (C) 2019 Cumulus Networks, Inc. All rights reserved

   @remark  This software is subject to the Cumulus Networks End User License
            Agreement available  at the following locations:

   @remark  Internet: https://cumulusnetworks.com/downloads/eula/latest/view/

   @remark  Cumulus Linux systems: /usr/share/cumulus/EULA.txt

*******************************************************************************/

#if !defined (__XPHY_H_)
#define __XPHY_H_

#include <appl/diag/shell.h>

/*------------------------------------------------------------------------------

                                 Constants

------------------------------------------------------------------------------*/

#define MAX_MDIO_ADDR       256


/*------------------------------------------------------------------------------

                                  Structures

------------------------------------------------------------------------------*/

/** @brief A structure to process a sub-option of the xphy command. Each entry
 *         specifies the sub-option and a function to call when that sub-option
 *         is found.
 */
typedef struct sub_options_s {
    char              * opt_str;
    cmd_result_t     (* handler)(int unit, args_t *args);
} sub_option_t;

/** @brief A structure of the "xphy init" command option values.
 */
typedef struct xphy_init_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  fw_load;
} xphy_init_opts_t;

typedef struct xphy_uninit_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
} xphy_uninit_opts_t;

typedef struct xphy_config_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  if_side;
    int  if_type;
    int  auto_neg;
    int  fec;
    int  speed;
} xphy_config_opts_t;

typedef struct xphy_pre_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  if_side;
    int  pre;
    int  pre2;
    int  main;
    int  post;
    int  post2;
    int  post3;
    int  amp;
} xphy_pre_opts_t;

typedef struct xphy_show_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  if_side;
    int  section;
} xphy_show_opts_t;

typedef struct xphy_polarity_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  if_side;
    int  direction;
    int  polarity;
} xphy_polarity_opts_t;

typedef struct xphy_ln_ctrl_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  if_side;
    int  direction;
    int  mode;
} xphy_ln_ctrl_opts_t;

typedef struct xphy_auto_neg_opts_s {
    int  phy_id;
    int  num_phys;
    int  lane_map;
    int  if_side;
    int  enable;
    int  master_ln;
    unsigned int tech_ability;
    unsigned int ext_tech_ability;
    unsigned int fec_ability;
    unsigned int pause_ability;
} xphy_auto_neg_opts_t;

typedef struct xphy_linkscan_opts_s {
    int  phy_id;
    int  lane_map;
    int  enable;
    bcm_port_t port;
    int  add;
    char *port_name;
    int  if_side;
    int  if_type;
    int  auto_neg;
    int  fec;
    int  speed;
} xphy_linkscan_opts_t;

typedef struct phy_functions_s phy_functions_t;
typedef struct phy_family_info_s phy_family_info_t;

struct phy_functions_s {
    int (* discover)(int phy_id, phy_family_info_t *phy_info, int unit);
    int (* init)(xphy_init_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* uninit)(xphy_uninit_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* config)(xphy_config_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* pre)(xphy_pre_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* polarity)(xphy_polarity_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* show)(xphy_show_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* ln_ctrl)(xphy_ln_ctrl_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* auto_neg)(xphy_auto_neg_opts_t *opts, phy_family_info_t *phy_info, int unit);
    int (* linkscan)(xphy_linkscan_opts_t *opts, phy_family_info_t *phy_info, int unit);
};

/** @brief A structure of phy family information */
struct phy_family_info_s {
    char *family_name;
    phy_functions_t * func;
};

typedef enum xphy_show_options_e {
    XPHY_SHOW_ALL,
    XPHY_SHOW_LINK_STATUS,
    XPHY_SHOW_MODE_CONFIG,
    XPHY_SHOW_POLARITY,
    XPHY_SHOW_VERSION,
    XPHY_SHOW_RX_PMD_LOCK,
    XPHY_SHOW_LOOPBACK,
    XPHY_SHOW_TX_PRE,
    XPHY_SHOW_RX_ANALOG,
    XPHY_SHOW_LANE_CONTROL,
    XPHY_SHOW_TX_TRAINING,
    XPHY_SHOW_CL73,
    XPHY_SHOW_EYE_SCAN,
    XPHY_SHOW_LANE_SWAP,
    XPHY_SHOW_FEC,
    XPHY_SHOW_DUMP,
    XPHY_SHOW_POWER,
    XPHY_SHOW_DIAGNOSTICS,
    XPHY_SHOW_LANE_CONFIG,
    XPHY_SHOW_LINKSCAN
} xphy_show_options_t;


/*------------------------------------------------------------------------------

                                  Global Data

------------------------------------------------------------------------------*/

extern char cmd_xphy_usage[];
extern char *if_options[];


/*------------------------------------------------------------------------------

                              Function Prototypes

------------------------------------------------------------------------------*/

extern int cmd_xphy(int unit, args_t *args);


#endif /** __XPHY_H_ */
